package ms.warpzone.warppay.manager;

import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.os.Handler;
import android.util.Log;
import android.widget.Toast;

import com.squareup.okhttp.Interceptor;
import com.squareup.okhttp.OkHttpClient;
import com.squareup.okhttp.Request;

import org.mindrot.jbcrypt.BCrypt;

import java.io.IOException;
import java.io.InputStream;
import java.security.Certificate;
import java.security.KeyManagementException;
import java.security.KeyStore;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.cert.CertificateException;
import java.security.cert.CertificateFactory;
import java.util.List;
import java.util.Timer;
import java.util.TimerTask;

import javax.net.ssl.SSLContext;
import javax.net.ssl.TrustManagerFactory;

import ms.warpzone.warppay.MainActivity;
import ms.warpzone.warppay.R;
import ms.warpzone.warppay.data.SQLiteService;
import ms.warpzone.warppay.data.models.local.Category;
import ms.warpzone.warppay.data.models.local.Product;
import ms.warpzone.warppay.data.models.local.User;
import ms.warpzone.warppay.data.RestService;
import ms.warpzone.warppay.data.models.rest.RestCategory;
import ms.warpzone.warppay.data.models.rest.RestProduct;
import ms.warpzone.warppay.data.models.rest.RestTransaction;
import ms.warpzone.warppay.data.models.rest.RestUser;
import ms.warpzone.warppay.dialogs.NyanCatDialog;
import ms.warpzone.warppay.dialogs.PinCodesDialog;
import ms.warpzone.warppay.orderList.Order;
import retrofit.Callback;
import retrofit.GsonConverterFactory;
import retrofit.Response;
import retrofit.Retrofit;

public class MainManager {

    private RestService restService;
    private MainActivity mainActivity;
    private SQLiteService sqLiteService;
    private UiManager uiManager;
    private DataManager dataManager;
    private boolean barcodeLearning=false;
    private Timer timer;
    private static MainManager instance = new MainManager();
    private Timer screenSaverTimer;

    public static MainManager getInstance() {
        return instance;
    }

    private MainManager() {
    }

    private RestService createRestService() {

        Retrofit.Builder builder = new Retrofit.Builder()
                .baseUrl("https://infra-test.warpzone/api/")
        .addConverterFactory(GsonConverterFactory.create());


        OkHttpClient okHttp = new OkHttpClient();
        try {
            okHttp.setSslSocketFactory(getSSLConfig(this.getMainActivity().getBaseContext()).getSocketFactory());
            okHttp.networkInterceptors().add(new Interceptor() {
                @Override
                public com.squareup.okhttp.Response intercept(Chain chain) throws IOException {
                    Request request = chain.request();
                    Request newRequest;

                    newRequest = request.newBuilder()
                            .addHeader("Authorization", "Token 7b6ca8ade59a32cef9b516e91f3bc9545b6c75a7")
                            .build();

                    return chain.proceed(newRequest);
                }
            });
        } catch (CertificateException | NoSuchAlgorithmException | KeyManagementException | IOException | KeyStoreException e) {
            e.printStackTrace();
        }

        Retrofit retrofit = builder.client(okHttp).build();

        return retrofit.create(RestService.class);
    }

    private static SSLContext getSSLConfig(Context context) throws CertificateException, IOException,
            KeyStoreException, NoSuchAlgorithmException, KeyManagementException {

        CertificateFactory cf = null;
        String password = "e44r4dv9z1d0vwr9erotafxe66114v31jwhjlvttc8qkoa3nrskcj4ml0pwrh8aw";
        KeyStore keyStore   = KeyStore.getInstance("BKS");
        keyStore.load(context.getResources().openRawResource(R.raw.keystore), password.toCharArray());

        String tmfAlgorithm = TrustManagerFactory.getDefaultAlgorithm();
        TrustManagerFactory tmf = TrustManagerFactory.getInstance(tmfAlgorithm);
        tmf.init(keyStore);

        SSLContext sslContext = SSLContext.getInstance("TLS");
        sslContext.init(null, tmf.getTrustManagers(), null);

        return sslContext;
    }
    public void init(MainActivity mainActivity) {
        this.mainActivity = mainActivity;
        this.sqLiteService = SQLiteService.getInstance();
        this.uiManager = UiManager.getInstance();
        this.dataManager = DataManager.getInstance();
        this.uiManager.initUi(this.mainActivity);

        this.restService = this.createRestService();
        this.refreshData();
    }

    public void refreshData() {
        this.restService.getAllUser().enqueue(new Callback<List<RestUser>>() {
            @Override
            public void onResponse(Response<List<RestUser>> response, Retrofit retrofit) {
                List<User> userList = MainManager.getInstance().sqLiteService.refreshUserData(response.body());
                if (userList != null) {
                    MainManager.getInstance().uiManager.refreshUserData(userList);
                }
                Toast.makeText(MainManager.getInstance().mainActivity, "Refresh successful", Toast.LENGTH_LONG).show();
            }

            @Override
            public void onFailure(Throwable t) {
                Log.d("REST", t.getMessage());
            }
        });

        this.restService.getAllCategories().enqueue(new Callback<List<RestCategory>>() {
            @Override
            public void onResponse(Response<List<RestCategory>> response, Retrofit retrofit) {
                List<Category> categoryList = MainManager.getInstance().sqLiteService.refreshCategoryData(response.body());
                if (categoryList != null) {
                    MainManager.getInstance().uiManager.showCategoryButtons();
                }
                Toast.makeText(MainManager.getInstance().mainActivity, "Refresh successful", Toast.LENGTH_LONG).show();
            }

            @Override
            public void onFailure(Throwable t) {
                Log.d("REST", t.getMessage());
            }
        });

        this.restService.getAllProducts().enqueue(new Callback<List<RestProduct>>() {
            @Override
            public void onResponse(Response<List<RestProduct>> response, Retrofit retrofit) {
                List<ms.warpzone.warppay.data.models.local.Product> productList = MainManager.getInstance().sqLiteService.refreshProductData(response.body());
                if (productList != null) {
                    MainManager.getInstance().uiManager.refreshProductData(productList);
                }
            }

            @Override
            public void onFailure(Throwable t) {
                Log.d("REST", t.getMessage());
            }
        });

    }

    public void setCurrentUser(User user) {
        this.setCurrentUser(user,false);
    }

    public void setCurrentUser(User user, boolean authorized) {
        if (user != null) {
            if(!user.getPinCode().isEmpty() && !authorized) {
                PinCodesDialog p = new PinCodesDialog(user);
                p.setOnDismissListener(new DialogInterface.OnDismissListener() {
                    @Override
                    public void onDismiss(DialogInterface dialogInterface) {
                        if(MainManager.getInstance().getCurrentUser() == null)
                            MainManager.getInstance().clearCurrentUser();
                    }
                });
                p.show();
                return;
            }
            this.dataManager.setCurrentUser(user);
            this.uiManager.setCurrentUser(user);
            this.startTimer();
        }
    }

    public void stopTimer() {
        if (this.timer != null) {
            this.timer.cancel();
            this.timer.purge();
        }
    }

    public void startTimer() {
        this.stopTimer();
        this.timer = new Timer();
        TimerTask t = new TimerTask() {
            @Override
            public void run() {
                Handler mainHandler = new Handler(MainManager.getInstance().getMainActivity().getApplicationContext().getMainLooper());
                Runnable myRunnable = new Runnable() {
                    @Override
                    public void run() {
                        MainManager.getInstance().clearCurrentUser();
                    }
                };
                mainHandler.post(myRunnable);
            }
        };

        this.timer.schedule(t, 30000);
    }

    public void resetScreenSaverTimer() {
        this.stopScreenSaverTimer();
        this.screenSaverTimer = new Timer();

        TimerTask t = new TimerTask() {
            @Override
            public void run() {
                Handler mainHandler = new Handler(MainManager.getInstance().getMainActivity().getApplicationContext().getMainLooper());
                Runnable myRunnable = new Runnable() {
                    @Override
                    public void run() {
                        new NyanCatDialog().show();
                    }
                };
                mainHandler.post(myRunnable);
            }
        };
        this.screenSaverTimer.schedule(t, 4000);
    }

    public void stopScreenSaverTimer() {
        if (this.screenSaverTimer != null) {
            this.screenSaverTimer.cancel();
            this.screenSaverTimer.purge();
        }
    }
    public User getCurrentUser() {
        return this.dataManager.getCurrentUser();
    }

    public void clearCurrentUser() {
        DataManager.getInstance().setLastCardId("");
        this.dataManager.clearCurrentUser();
        this.uiManager.clearCurrentUser();
        this.stopTimer();
    }

    public void addOrder(Order order) {
        DataManager.getInstance().addOrder(order);
        this.uiManager.refreshTotalTextView(DataManager.getInstance().getTotalAmount());
    }

    public void removeOrder(Order order) {
        DataManager.getInstance().removeOrder(order);
        this.uiManager.removeOrder(order);
        this.uiManager.refreshTotalTextView(DataManager.getInstance().getTotalAmount());

    }
    public void chargeAmount(double amount) {
        User currentUser = this.dataManager.getCurrentUser();
        RestTransaction t = new RestTransaction();
        t.setAmount(amount);
        t.setTrans_type(1);
        t.setCash_paid(true);
        MainManager.getInstance().getRestService().saveTransaction(currentUser.getUserid(), t).enqueue(new Callback<Void>() {
            @Override
            public void onResponse(Response<Void> response, Retrofit retrofit) {
            }

            @Override
            public void onFailure(Throwable t) {
                Log.d("REST", t.getMessage());
            }
        });
        MainManager.getInstance().getRestService().getSingleUser(currentUser.getUserid()).enqueue(new Callback<RestUser>() {
            @Override
            public void onResponse(Response<RestUser> response, Retrofit retrofit) {
                DataManager.getInstance().getCurrentUser().setCredit(response.body().getCredit());
                MainManager.getInstance().uiManager.refreshCreditTextView(DataManager.getInstance().getCurrentUser().getCredit());
            }

            @Override
            public void onFailure(Throwable t) {
                Log.d("REST", t.getMessage());
            }
        });


    }
    public void performPayment(Boolean cash) {
        User user = DataManager.getInstance().getCurrentUser();
        double totalAmount = DataManager.getInstance().getTotalAmount();

        if (totalAmount > 0.0 && user != null){
            if (DataManager.getInstance().getCurrentUser().getCredit()>=totalAmount || cash){
                DataManager.getInstance().performPayment(cash);
                MainManager.getInstance().clearCurrentUser();

            } else {
                Toast.makeText(mainActivity, mainActivity.getResources().getString(R.string.please_charge), Toast.LENGTH_LONG).show();
            }
        }
    }


    public MainActivity getMainActivity() {
        return mainActivity;
    }

    public void setMainActivity(MainActivity mainActivity) {
        this.mainActivity = mainActivity;

    }

    public RestService getRestService() { return restService; }


    public void saveCardId() {
        final String card_id = this.dataManager.getLastCardId();

        if (card_id != null) {
            new AlertDialog.Builder(this.mainActivity)
                .setTitle(this.mainActivity.getResources().getString(R.string.confirm_learn_card))
                .setMessage("Die karten id "+card_id+" wird dem Nutzer "+this.getCurrentUser()+" hinzugefügt")
                .setPositiveButton(android.R.string.yes, new DialogInterface.OnClickListener() {
                    public void onClick(DialogInterface dialog, int which) {
                        MainManager.getInstance().getCurrentUser().setCardId(card_id);
                        MainManager.getInstance().dataManager.saveCurrentUser();

                    }
                })
                .setNegativeButton(android.R.string.no, new DialogInterface.OnClickListener() {
                    public void onClick(DialogInterface dialog, int which) {
                    }
                })
                .setIcon(android.R.drawable.ic_dialog_alert)
                .show();
        }
    }

    public boolean isBarcodeLearning() {
        return barcodeLearning;
    }

    public void setBarcodeLearning(boolean barcodeLearning) {
        this.barcodeLearning = barcodeLearning;
    }
}
